// C Source File
// Created 10/10/2003; 9:25:14 PM
#include "All.h"

typedef struct {
	unsigned short Size;
	config Config;
	char Zero1;
	char Type[3];
	char Zero2;
	char Tag;
} Save;

// Saves the Difficulty and Time Limit Settings
void Save_Config(void) {
	//unsigned char Extension[] = { 0, 'c', 'f', 'g', 0, OTH_TAG };
	char *Base;
	HANDLE h;
	Save Save_File;
	if (Config->Speed < 1 || Config->Speed > 15)
		Config->Speed = 7;
	
	Save_File.Size  = sizeof(Save) - 2;
	memcpy(&Save_File.Config, Config, sizeof(config));
	Save_File.Zero1 = 0;
	sprintf(Save_File.Type, "cfg");
	Save_File.Zero2 = 0;
	Save_File.Tag = OTH_TAG;
	
	UnArchive("hockycfg");  // Make sure that before writing to the file, it is not archived
	
	if ((h = HeapAlloc(sizeof(Save))) == H_NULL)
		return;
	if ((Base = HeapDeref(DerefSym(SymAdd(SYMSTR("hockycfg")))->handle = h)) == NULL) {
		HeapFree(h);
    return;
	}
	memcpy(Base, &Save_File, sizeof(Save));
	
	Archive("hockycfg");    // Automatically Archive the config file in case of a crash
}

// Tries to Load an external Hockey Configuration File
void Load_Config(void) {
	Config->Difficulty = FALSE;  // Default Settings
	Config->Time_Limit = 3;
	Config->Speed      = 7;
	unsigned char *Ptr = Get_Data_Ptr("hockycfg", 0);
	if (Ptr == NULL)
		return;
	
	memcpy(Config, Ptr, sizeof(config));
	if (Config->Speed < 1 || Config->Speed > 15)
		Config->Speed = 7;
}

// Loads all of the Team Data from the External File "hockteam"
void Load_Teams(void) {
	unsigned char *TeamPtr = NULL;
	
	// If there is not an external hockteam File, then just set the default teams
	Set_Default_Teams(&TeamPtr);
	if (TeamPtr == NULL) {  // Teams need to be loaded
		if ((TeamPtr = Get_Data_Ptr("hockteam", 0)) == NULL)
			ERROR(EXTERNAL_FILE_HOCKTEAM);  // Error Finding hockteam File Pointer
		game->No_Teams[0] = *TeamPtr++;
		game->No_Teams[1] = *TeamPtr++;
	}
	
	if ((Western = malloc(sizeof(Names[game->No_Teams[0]]))) == NULL)
		ERROR(MEMORY);
	memcpy(Western, TeamPtr, sizeof(Names[game->No_Teams[0]]));
	TeamPtr += sizeof(Names[game->No_Teams[0]]);
	if ((Eastern = malloc(sizeof(Names[game->No_Teams[1]]))) == NULL)
		ERROR(MEMORY);
	memcpy(Eastern, TeamPtr, sizeof(Names[game->No_Teams[1]]));
}

// Checks to see if there is already a "hockteam" file and if there isn't, set the default
void Set_Default_Teams(unsigned char **TeamPtr) {
	SYM_ENTRY *SymPtr;
	
	if ((SymPtr = SymFindPtr(SYMSTR("hockteam"), 0)) == NULL) {  // Hockteam file not found
		// The Default Teams need to be loaded
		*TeamPtr = (unsigned char *)Default_Western;
		game->No_Teams[0] = 16;
		game->No_Teams[1] = 15;
	}
}

// Saves all of the Team Data into the External File "hockteam"
void Save_Teams(void) {
	short Size = sizeof(Names[game->No_Teams[0]]) + sizeof(Names[game->No_Teams[1]]) + 11;
	char *Base;
	HANDLE h;
	struct {
		unsigned short Size;
		unsigned char No1;
		unsigned char No2;
		Names Western[game->No_Teams[0]];
		Names Eastern[game->No_Teams[1]];
		char Zero1;
		char Type[4];
		char Zero2;
		char Tag;
	} Save_File;
	
	Save_File.Size  = Size - 2;
	Save_File.No1   = game->No_Teams[0];
	Save_File.No2   = game->No_Teams[1];
	memcpy(&Save_File.Western, Western, sizeof(Names[game->No_Teams[0]]));
	memcpy(&Save_File.Eastern, Eastern, sizeof(Names[game->No_Teams[1]]));
	Save_File.Zero1 = 0;
	sprintf(Save_File.Type, "Team");
	Save_File.Zero2 = 0;
	Save_File.Tag = OTH_TAG;
	
	UnArchive("hockteam");  // Make sure that before writing to the file, it is not archived
	
	if ((h = HeapAlloc(Size)) == H_NULL) {
		Archive("hockteam");
		return;
	} if ((Base = HeapDeref(DerefSym(SymAdd(SYMSTR("hockteam")))->handle = h)) == NULL) {
		HeapFree(h);
    return;
	}
	memcpy(Base, &Save_File, Size);
	
	Archive("hockteam");    // Automatically Archive the team file in case of a crash
}

// Used by Load_Teams(); Returns a Pointer to the Data of a File
unsigned char *Get_Data_Ptr(const char *Filename, short Offset) {
	unsigned char *Location;
	SYM_ENTRY *SymPtr;
	HANDLE h;
	
	if ((SymPtr = SymFindPtr(SYMSTR(Filename), 0)) == NULL)
		return NULL;  // File not found
	
	h = SymPtr->handle;
	
	if ((Location = (unsigned char*) HeapDeref(h)) == NULL)
		return NULL;
	return (Location + 2 + Offset);
}

void Archive(const char *Filename) {
	SYM_ENTRY *SymPtr = NULL;
	HSym hsym = SymFind(SYMSTR(Filename));
	
	SymPtr = DerefSym (hsym);
	
	if (EM_findEmptySlot(*Get_Data_Ptr(Filename, -2)) == NULL)
		return;  // Make sure Garbage Collection will not occur
	if (!SymPtr->flags.bits.archived)
		EM_moveSymToExtMem(SYMSTR(Filename), HS_NULL);
}

void UnArchive(const char *Filename) {
	SYM_ENTRY *SymPtr = NULL;
	HSym hsym = SymFind(SYMSTR(Filename));
	
	SymPtr = DerefSym (hsym);
	
	if (SymPtr->flags.bits.archived)
		EM_moveSymFromExtMem(SYMSTR(Filename), HS_NULL);
}

#ifdef COMPRESSION

/* Decompresses the RLE Vertically Compressed Titlescreen */
void RLE_Decompress(unsigned char *src, unsigned char *dest, short size)
{
	int i = 0, j;
	int pos = 0;
	int dcmpNum = 0;
	
	while (dcmpNum < size)
	{
		//decompress a run
		if (src[i] == 0x91)
		{
			i++;
			for(j = 0; j < src[i + 1]; j++)
			{
				dest[pos] = src[i];
				pos += 20;
				if (pos > 20 * 100)
				{
					pos -= 20 * 100;
					pos++;
				}
			}
			i++;
			dcmpNum += src[i];
		}
		else
		{
			dest[pos] = src[i];
			pos += 20;
			if (pos > 20 * 100)
			{
				pos -= 20 * 100;
				pos++;
			}
			dcmpNum++;
		}
		i++;
	}
}
#endif

/*void Save_Tournament(void) {
	char *Base, FileName[8] = "hockys";
	HANDLE h;
	
	struct {
		unsigned char Size;
		char Score[2];
		char Teams[2][2];
		char T_Board[8][3];
		char Round;
		char Difficulty;
		char Seconds;
		char Minutes;
		char Zero1;
		char Type[3];
		char Zero2;
		char Tag;
	} Save_File;
}*/